<?php

namespace Drupal\xntttsv\Plugin\ExternalEntities\StorageClient;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\external_entities\ResponseDecoder\ResponseDecoderFactoryInterface;
use Drupal\xnttfiles\Plugin\ExternalEntities\StorageClient\FileClientBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * External entities storage client for TSV files.
 *
 * @ExternalEntityStorageClient(
 *   id = "xntttsv",
 *   label = @Translation("TSV Files"),
 *   description = @Translation("Retrieves and stores records in TSV files.")
 * )
 */
class TsvFiles extends FileClientBase {

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    TranslationInterface $string_translation,
    ResponseDecoderFactoryInterface $response_decoder_factory,
    ConfigFactory $config_factory,
    MessengerInterface $messenger,
    LoggerChannelInterface $logger,
    CacheBackendInterface $cache
  ) {
    parent::__construct(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $string_translation,
      $response_decoder_factory,
      $config_factory,
      $messenger,
      $logger,
      $cache
    );
    // Defaults.
    $this->fileType = 'TSV';
    $this->fileTypePlural = 'TSV';
    $this->fileTypeCap = 'TSV';
    $this->fileTypeCapPlural = 'TSV';
    $this->fileExtension = '.tsv';
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('string_translation'),
      $container->get('external_entities.response_decoder_factory'),
      $container->get('config.factory'),
      $container->get('messenger'),
      $container->get('logger.channel.xntttsv'),
      $container->get('cache.default')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function parseFile(string $file_path) :array {
    $data = [];
    $raw_tsv = file($file_path);
    if (!empty($raw_tsv)) {
      // Get stored field names (TSV first line).
      do {
        $column_line = array_shift($raw_tsv);
        // Skip comments and empty lines.
        // @todo: maybe add a plugin setting to allow comments or not.
      } while (isset($column_line) && preg_match('/^#|^\s*$/', $column_line));
      if (!isset($column_line)) {
        return [];
      }
      // Remove EOL char.
      $column_line = substr($column_line, 0, -1);
      $data[''] = $columns = explode("\t", $column_line);
      // Iterate on entity lines.
      $idf = $this->getIdField();
      foreach ($raw_tsv as $raw_line) {
        if (preg_match('/^#|^\s*$/', $raw_line)) {
          // Skip comments and empty lines.
          continue;
        }
        // Extract value and also remove EOL char.
        $values = explode("\t", substr($raw_line, 0, -1));
        $entity = array_combine($columns, $values);
        $data[$entity[$idf]] = $entity;
      }
    }
    return $data;
  }

  /**
   * {@inheritdoc}
   */
  protected function generateRawData(array $entities_data) :string {
    $raw_data = '';
    // Write columns.
    $columns = $entities_data[''];
    unset($entities_data['']);
    $raw_data .= implode("\t", $columns) . "\n";
    // Write other records.
    foreach ($entities_data as $id => $entity) {
      $entity_data = [];
      foreach ($columns as $field) {
        $entity_data[] = str_replace(
          ["\\", "\n", "\t", "\r"],
          ['\\\\', '\n', '\t', '\r'],
          $entity[$field] ?? ''
        );
      }
      $raw_data .= implode("\t", $entity_data) . "\n";
    }
    return $raw_data;
  }

}
